--******************************************************************************
--*
--*  s_Color struct
--*  by Ofer Zelichover (c) 12/2004
--*  http://tdp.nu/ofer/ ; ofer_z@hotmail.com
--*
--******************************************************************************
--*  You may use this script freely as you see fit.
--*  You may use parts or the script as a whole in your own scripts.
--*  (it would be nice if you give me a creadit if you do so ;))
--******************************************************************************
--*  This script comes with no warranty.
--*  I cannot be held responsible for any loss of data, time or any other
--*  damage or loss due to the use of this code.
--*
--*  ********************************************************************
--*  *** IF YOU DON'T LIKE THE ABOVE STATEMENT, DON'T USE THIS SCRIPT ***
--*  ********************************************************************
--*
--*  **** This script was written for max 7 and wasn't tested on other   ****
--*  **** vertions of max.                                               ****
--*
--*  If you find any bugs in it please let me know.
--******************************************************************************
--* Description
--* -------------
--*  this a struct for color conversions. written for use in ACP.
--*
--******************************************************************************



struct s_color (
	colorValue = black,
	previousHue, previousSat, previousVal,
	rgbToLumRatio = [0.30, 0.59, 0.11],
	
	
fn fromRGB r g b = 
	(
		colorValue = color r g b
	),


		fn toRGB &r &g &b = 
	(
		r = colorValue.r
		g = colorValue.g
		b = colorValue.b
		#(r, g, b)
	),
	
	fn fromHSV  H S V ignorePrevious:false = 
	(
		if not ignorePrevious then (
			if V == 0 and previousHue == undefined and previousSat == undefined then (
				previousHue = colorValue.h
				previousSat = colorValue.s
			)
	
			if V != 0 and previousHue != undefined and previousSat != undefined then 
(
				H = previousHue
				S = previousSat
			)
	
			if S == 0 and previousHue == undefined and previousVal == undefined then (
				previousHue = colorValue.h
				previousVal = colorValue.v
			)
	
			if S != 0 and previousHue != undefined and previousVal != undefined then 
(
				H = previousHue
				V = previousVal
			)
		)
	
		local col = red
		col.s = S
		col.v = V
		col.h = H

		if V != 0 and S != 0 then (
			previousHue = undefined
			previousSat = undefined
			previousVal = undefined
		)

		colorValue = col
	),
	
	
fn toHSV &h &s &v =
	(
		h = colorValue.h
		s = colorValue.s
		v = colorValue.v
		#(h, s, v)
	),
	
	-- Lab conversion XYZref
	XYZref = #(
		-- Observer 2 (CIE 1931)
		[109.850,	100.0,	35.585],	-- A (Incandescent)
		[98.074,	100.0,	118.232],	-- C
		[96.422,	100.0,	82.521],	-- D50
		[95.682,	100.0,	92.149],	-- D55
		[95.047,	100.0,	108.883],	-- D65 (Daylight)
		[94.972,	100.0,	122.638],	-- D75
		[99.187,	100.0,	67.395],	-- F2 (Flourescent)
		[95.044,	100.0,	108.755],	-- F7
		[100.966,	100.0,	64.370],	-- F11
		-- Observer 10 (CIE 1964)
		[111.144,	100.0,	35.200],	-- A (Incandescent)
		[97.285,	100.0,	116.145],	-- C
		[96.720,	100.0,	81.427],	-- D50
		[95.799,	100.0,	90.926],	-- D55
		[94.811,	100.0,	107.304],	-- D65 (Daylight)
		[94.416,	100.0,	120.641],	-- D75
		[103.280,	100.0,	69.026],	-- F2 (Flourescent)
		[95.792,	100.0,	107.687],	-- F7
		[103.866,	100.0,	65.627]		-- F11
	),



	
fn fromXYZ X Y Z =
	(
		X /= 100.
		Y /= 100.
		Z /= 100.
		local _r = X*3.2310 + Y*-1.5374 + Z*-0.4986
		local _g = X*-0.9692 + Y*1.8760 + Z*0.0416
		local _b = X*0.0556 + Y*-0.2040 + Z*1.0570
		_r = if _r > 0.00304 then (1.055*(_r^(1/2.4))-0.055) else (12.92*_r)
		_g = if _g > 0.00304 then (1.055*(_g^(1/2.4))-0.055) else (12.92*_g)
		_b = if _b > 0.00304 then (1.055*(_b^(1/2.4))-0.055) else (12.92*_b)
		colorValue = color (_r*255) (_g*255) (_b*255)
	),
	
	
fn toXYZ &X &Y &Z =
	(
		local _r = colorValue.r/255.
		local _g = colorValue.g/255.
		local _b = colorValue.b/255.
		_r = 100. * (if _r > 0.03928 then (((_r+0.055)/1.055)^2.4) else (_r/12.92))
		_g = 100. * (if _g > 0.03928 then (((_g+0.055)/1.055)^2.4) else (_g/12.92))
		_b = 100. * (if _b > 0.03928 then (((_b+0.055)/1.055)^2.4) else (_b/12.92))
		X = _r*0.4124 + _g*0.3576 + _b*0.1805
		Y = _r*0.2126 + _g*0.7152 + _b*0.0722
		Z = _r*0.0193 + _g*0.1192 + _b*0.9505
		#(X, Y, Z)
	),
	
	-- converts rgb values into Lab
	fn toLab &L &a &b ref:5 = (
		-- convert RGB -> XYZ
		local _x, _y, _z
		
toXYZ &_x &_y &_z
		
		--convert XYZ -> Lab
		_x /= XYZref[ref].x
		_y /= XYZref[ref].y
		_z /= XYZref[ref].z
		_x = if _x > 0.008856 then (_x^(1/3.)) else ((7.787*_x)+(16/116.))
		_y = if _y > 0.008856 then (_y^(1/3.)) else ((7.787*_y)+(16/116.))
		_z = if _z > 0.008856 then (_z^(1/3.)) else ((7.787*_z)+(16/116.))
		L = (116 * _y) - 16
		a = 500 * (_x - _y)
		b = 200 * (_y - _z)
		#(L, a, b)
	),

	-- converts Lab values into RGB
	fn fromLab L a b ref:5= (
		-- convert Lab -> XYZ
		local _y = (L + 16) / 116.
		local _x = (a / 500.) + _y
		local _z = _y - (b / 200.)
		_y = XYZref[ref].y * (if _y^3 > 0.008856 then (_y^3) else (((_y-16)/116.)/7.787))
		_x = XYZref[ref].x * (if _x^3 > 0.008856 then (_x^3) else (((_x-16)/116.)/7.787))
		_z = XYZref[ref].z * (if _z^3 > 0.008856 then (_z^3) else (((_z-16)/116.)/7.787))
		
		-- convert XYZ -> RGB
		fromXYZ _x _y _z
	),
	
	-- converts hexadecimal number to integer
	fn HexToDec n = 
	(
		try((execute ("0x"+n)))catch(0)
	),


		fn fromHex h =
	(
		local n = hexToDec h
		local b = mod n 256
		n = n/256 as integer
		local g = mod n 256
		n = n/256 as integer
		local r = mod n 256
		fromRGB r g b
	),

	fn leadingZero num len =
	(
		local t = num as string
		while t.count < len do t = "0" + t
		t
	),

	-- converts decimal number to hexadecimal
	fn DecToHex n = 
	(
		local hexArray = #("0","1","2","3","4","5","6","7","8","9","A","B","C","D","E","F")
		if n<16 then (
			if n < 0 or (n+1) > hexArray.count then
				"0"
			else
				hexArray[n+1]
		) else (
			local nn = mod n 16 as integer
			( (DecToHex (n/16 as integer)) + hexArray[nn+1] )
		)
	),
	
	fn toHex = 
	(
		local s = leadingZero (DecToHex colorValue.r) 2
		s += leadingZero (DecToHex colorValue.g
) 2
		s += leadingZero (DecToHex colorValue.b) 2
		s
	),
	
	
	
-- CMYK convertions
	fn fromCMY C M Y =
	(
		local r = (1. - C) * 255
		local g = (1. - M) * 255
		local b = (1. - Y) * 255
		fromRGB r g b
	),
			
	fn toCMY &C &M &Y =
	(
		C = 1. - (colorValue.r / 255.)
		M = 1. - (colorValue.g / 255.)
		Y = 1. - (colorValue.b / 255.)
		#(C, M, Y)
	),
	
	fn fromCMYK C M Y K =
	(
		local _C = ( C * ( 1 - K ) + K )
		local _M = ( M * ( 1 - K ) + K )
		local _Y = ( Y * ( 1 - K ) + K )

		fromCMY _C _M _Y
	),

	fn toCMYK &C &M &Y &K =
	(
		local _K = 1.
		local _C, _M, _Y
		
		toCMY &_C &_M &_Y
		
		if _C < _K then _K = _C
		if _M < _K then _K = _M
		if _Y < _K then _K = _Y
		
		C = (_C - _K) / (1. - _K) 
		M = (_M - _K) / (1. - _K) 
		Y = (_Y - _K) / (1. - _K)
		K = _K
	),
	
	
fn toLum =
	(
		((colorValue.r * rgbToLumRatio[1]) + (colorValue.g * rgbToLumRatio[2]) + (colorValue.b * rgbToLumRatio[3]))
	),
	
	fn invert =
	(
		[255,255,255] - colorValue
	)
	
) -- end of s_color struct


