--******************************************************************************
--*
--*  s_Palette struct
--*  by Ofer Zelichover (c) 12/2004
--*  Last update: 4:28 PM Tuesday, June 12, 2007
--*  http://www.oferz.com  ; maxscripts@oferz.com
--*
--******************************************************************************
--*  You may use this script freely as you see fit.
--*  You may use parts or the script as a whole in your own scripts.
--*  (it would be nice if you give me a creadit if you do so ;))
--******************************************************************************
--*  This script comes with no warranty.
--*  I cannot be held responsible for any loss of data, time or any other
--*  damage or loss due to the use of this code.
--*
--*  ********************************************************************
--*  *** IF YOU DON'T LIKE THE ABOVE STATEMENT, DON'T USE THIS SCRIPT ***
--*  ********************************************************************
--*
--*  **** This script was written for max 7 and wasn't tested on other   ****
--*  **** vertions of max.                                               ****
--*
--*  If you find any bugs in it please let me know.
--******************************************************************************
--* Description
--* -------------
--*  this a struct for palette management. written for colorPalette
--*  support save/load of palette, sort, search, and various basic management functions
--*
--******************************************************************************



struct s_swatch (
	color = black,
	name = ""
)

struct s_palette (
	swatches = #(),
	width = 100,
	height = 21,
	scroll = 1,
	selected,
	swatchHeight = 12,
	selectedColor1 = red * 0.5,
	selectedColor2 = yellow,
	spacing = 2,
	spacingColor = black,
	bm,
	
	fn generatePalette =
	(
		bm = bitmap (width + (2 * spacing) + 2) (height * (swatchHeight + spacing) + spacing) color:spacingColor
		local sw, ii, yy
		local colArray = #()
		for i = 1 to height do (
			ii = scroll + i - 1
			sw = if (ii <= swatches.count) and (ii > 0) then swatches[ii] else s_swatch()
			colArray = for x = 1 to width collect sw.color
			yy = ((i - 1) * (swatchHeight + spacing)) + spacing
			for y = 0 to (swatchHeight - 1) do (
				setPixels bm [(spacing + 1), (yy + y)] colArray
			)
		)
		bm
	),
	
	fn drawBorder pos color1 color2 =
	(
		if not (isKindOf pos Integer) or pos < 1 or pos > swatches.count then return()
		
		local arr = for i = 1 to (width + 2) collect if (mod i 2) == 1 then color1 else color2
		local x = spacing
		local y = ((pos - 1) * (swatchHeight + spacing)) + spacing - 1
		
		for i = 1 to (swatchHeight + 2) do (
			if i == 1 or i == (swatchHeight + 2) then
				setPixels bm [x, (y + i - 1)] arr
			else (
				setPixels bm [x, (y + i - 1)] #(if (mod i 2) == 1 then color1 else color2)
				setPixels bm [(x + width + 1), (y + i - 1)] #(if (mod i 2) == 1 then color1 else color2)
			)				
		) 
	),
	
	fn markSelected =
	(
		if not (isKindOf selected Integer) or selected < 1 or selected > swatches.count then return()
		drawBorder (selected - scroll + 1) selectedColor1 selectedColor2
	),
	
	fn unMarkSelected = 
	(
		if (isKindOf selected Integer) and selected > 0 and selected <= swatches.count then
			drawBorder (selected - scroll + 1) spacingColor spacingColor
	),
	
	fn centerSelected =
	(
		if not (isKindOf selected Integer) or selected < 1 or selected > swatches.count then return()
		scroll = amax (amin (selected - (height / 2)) (swatches.count - height + 1)) 1
		generatePalette()
	),


	fn select swatch center:false =
	(
		unMarkSelected()
		selected = swatch
		if center then 
			centerSelected()
		markSelected()
	),
	
	fn deSelect =
	(
		unMarkSelected()
		selected = undefined
	),
	
	fn getSwatchName swatch =
	(
		if not (isKindOf swatch Integer) or swatch < 1 or swatch > swatches.count then return()
		if swatches[swatch].name != undefined then swatches[swatch].name as string
		else ""
	),

	fn setSwatchName swatch name =
	(
		if not (isKindOf swatch Integer) or swatch < 1 or swatch > swatches.count then return()
		if swatches[swatch] != undefined then 
			swatches[swatch].name = name as string
	),


	fn getSwatchColor swatch =
	(
		if not (isKindOf swatch Integer) or swatch < 1 or swatch > swatches.count then return()
		swatches[swatch].color
	),
	
	fn setSwatchColor swatch color =
	(
		if not (isKindOf swatch Integer) or swatch < 1 or swatch > swatches.count then return()
		if swatches[swatch] != undefined then 
			swatches[swatch].color = color
	),


	fn isValidIndex index =
	(
		index <= swatches.count and index > 0
	),

	fn getSwatchIndex pos =
	(
		if not (isKindOf pos Point2) or not (isKindOf bm BitMap) then return()
		local p = pos
		p.x = amax 1 p.x
		p.x = amin (bm.width-spacing-1) p.x
		p.y = amax 1 p.y
		p.y = amin (bm.height-spacing-1) p.y
		local row = (p.y - spacing) / (swatchHeight + spacing)
		((row as integer) + scroll as integer)
	),
	
	fn addSwatch name color =
	(
		append swatches (s_swatch name:name color:color)
	),
	
	fn deleteSwatch swatch =
	(
		if not (isKindOf swatch Integer) or swatch < 1 or swatch > swatches.count then return()
		deleteItem swatches (swatch)
	),
	

	fn moveSwatch index newIndex =
	(
		if not isValidIndex index or not isValidIndex newIndex then return false
		local item = swatches[index]
		deleteSwatch index
		insertItem item swatches newIndex
		true
	),
	
	fn reset =
	(
		swatches = #()
	),

	fn bytesToShort i1 i2 =
	(
		(i1 * 256) + i2
	),
	
	fn swapBytes i =
	(
		i = i as integer
		local il = (mod i 256) as integer
		local ih = i / 256
		(il * 256) + ih
	),
	

	fn loadPalette file showWarnings:true appendPalette:false =
	(
		if not isKindOf file String then return()

		local colors = #()
		fn readColorSwatch f ver nc &warnings &colors =
		(
			colors = #()
				if ver != 1 and ver != 2 then return false
			local space, w, x, y, z, nameLen, nameStr
			local col = s_color()
			local sign_w = sign_x = sign_y = sign_z = #unsigned
			for i = 1 to nc do (
				space = bytesToShort (readByte f #unsigned) (readByte f #unsigned)
				case space of (
					1: append warnings 1;
					2: append warnings 2;
					7: (append warnings 7; sign_w = #unsinged; sign_x = #singed; sign_y = #singed; sign_z = #unsinged;)
					9: append warnings 9;
				)
				w = bytesToShort (readByte f sign_w) (readByte f sign_w)
				x = bytesToShort (readByte f sign_x) (readByte f sign_x)
				y = bytesToShort (readByte f sign_y) (readByte f sign_y)
				z = bytesToShort (readByte f sign_z) (readByte f sign_z)
				if (ver == 2) then (
					readShort f
					nameLen = bytesToShort (readByte f #unsigned) (readByte f #unsigned)
					nameStr = ""
					for ii = 1 to (nameLen - 1) do (
						nameStr += bit.intAsChar (bytesToShort (readByte f #unsigned) (readByte f #unsigned))
					)
					readShort f
				)
				case space of (
					0: col.fromRGB (w/65535.*255) (x/65535.*255) (y/65535.*255)
					1: col.fromHSV (w/182.04) (x/655.35) (y/655.35)
					2: col.fromCMYK (1.-(w/65535.)) (1.-(x/65535.)) (1.-(y/65535.)) (1.-(z/65535.))
					7: col.fromLab (w/100.) (x/100.) (y/100.)
					8: col.fromRGB (255-(w/39.2157)) (255-(w/39.2157)) (255-(w/39.2157)) -- grayscale: only w is used
					9: col.fromCMYK (w/655.35) (x/655.35) (y/655.35) (z/655.35) -- documentation says wide CMYK, not sure if this my trasnformation here is correct.
				)
				append colors (s_swatch name:nameStr color:col.colorValue)
			)
			true
		) -- end of readColorSwatch fn
		
		if not doesFileExist file then return false
		local f = fopen file "rb"
		fseek f 0 #seek_end
		local fLength = ftell f
		fseek f 0 #seek_set
		local warnings = #{}
		try (
			while (ftell f) < fLength do (
				local ver = bytesToShort (readByte f #unsigned) (readByte f #unsigned)
				local nc = bytesToShort (readByte f #unsigned) (readByte f #unsigned) -- number of colors in the palette
				readColorSwatch f ver nc &warnings &colors
			)
		) catch(print "Error reading ACO file."; return false)	

		fclose f
		
		if appendPalette then (
			join swatches colors
		) else (
			swatches = colors
		)
		
		if showWarnings and warnings.count > 0 then (
			local str = "The palette that was loaded contains color\nin these color spaces:\n\n"
			case of (
				(warnings[1]): str += "    - HSV\n"
				(warnings[2]): str += "    - CMYK\n"
				(warnings[7]): str += "    - Lab\n"
				(warnings[9]): str += "    - Wide CMYK\n"
			)
			str += "\nThese color spaces are not guaranteed to be\n trasnlated correctly.\nPlease see the documentation that came with this\nscript for more details."

			messageBox str title:"Notice" beep:false
		)		
		
		true
	), -- end loadPalette function
	
	-- Save paletee in adobe aco file format. 
	-- NOTE: the palettes saved with this function save colors only in RGB color space!
	fn savePalette file =
	(
		if not isKindOf file String then return()
		local f = fopen file "wb"
		
		-- write version 1 part
		writeShort f 0x0100 #unsigned 						-- ver 1
		writeShort f (swapBytes swatches.count) #unsigned 	-- number of colors
		for i = 1 to swatches.count do (					-- write the colors:
			writeShort f 0x0000 #unsigned 					-- color space
			writeShort f (swapBytes ((getSwatchColor i).r/255. * 65535)) #unsigned 	-- color space
			writeShort f (swapBytes ((getSwatchColor i).g/255. * 65535)) #unsigned 	-- color space
			writeShort f (swapBytes ((getSwatchColor i).b/255. * 65535)) #unsigned 	-- color space
			writeShort f 0x0000 #unsigned 					-- unused in this color space
			
		)

		-- write version 2 part
		writeShort f 0x0200 #unsigned 						-- ver 1
		writeShort f (swapBytes swatches.count) #unsigned 	-- number of colors
		for i = 1 to swatches.count do (					-- write the colors:
			writeShort f 0x0000 #unsigned 					-- color space
			writeShort f (swapBytes ((getSwatchColor i).r/255. * 65535)) #unsigned 	-- color space
			writeShort f (swapBytes ((getSwatchColor i).g/255. * 65535)) #unsigned 	-- color space
			writeShort f (swapBytes ((getSwatchColor i).b/255. * 65535)) #unsigned 	-- color space
			writeShort f 0x0000 #unsigned 					-- unused in thin color space
			writeShort f 0x0000 #unsigned					-- always 0
			local nameStr = getSwatchName i

				local nameLen = nameStr.count
			writeShort f (swapBytes (nameLen + 1)) #unsigned 	-- name length + 1
			for ii = 1 to nameLen do (
				writeShort f (swapBytes (bit.charAsInt nameStr[ii])) #unsigned 	-- write name caracters
			)				
			writeShort f 0x0000 #unsigned					-- always 0
		)
		
		fclose f
	),
	
	fn getBitmapPalette bm =
	(
		local prevType = portable_network_graphics.ipngio.getType()
		portable_network_graphics.ipngio.setType #paletted
		local b = copy bm
		local fname = sysInfo.tempdir + "tmp" + (timeStamp() as string) + ".png"
		b.filename = fname
		save b
		b = openBitmap fname
		swatches = for i = 1 to b.palette.count collect (s_swatch name:((i-1) as string) color:b.palette[i])
		b = undefined
		deleteFile fname
		portable_network_graphics.ipngio.setType prevType
	),




-- Sort functions	
	fn arrayCompare a1 a2 =
	(
		for i = 1 to (amin (a1.count) (a2.count)) do (
			if a1[i] > a2[i] then return 1
			else if a1[i] < a2[i] then return -1
		)
		if a1.count > a2.count then 1
		else if a1.count < a2.count then -1
		else 0
	),
		
	fn sortByProp v1 v2 prop =
	(
		if not isProperty v1 prop or not isProperty v2 prop then return 0
		local p1 = getProperty v1 prop
		local p2 = getProperty v2 prop
		if prop == #color or (matchPattern prop pattern:"color" ignoreCase:true) then (
			if p1.r > p2.r then 1
			else if p1.r < p2.r then -1
			else if p1.g > p2.g then 1
			else if p1.g < p2.g then -1
			else if p1.b > p2.b then 1
			else if p1.b < p2.b then -1
			else 0
		) else (
			case of (
				(p1 == p2): 0
				(p1 > p2): 1
				(p1 < p2): -1
			)
		)
	),
	
	fn sortByName =
	(
		fn sortFn v1 v2 = (sortByProp v1 v2 "name")
		qsort swatches sortFn
	),
	
	fn sortByRGB =
	(
		fn sortFn v1 v2 = (sortByProp v1 v2 "color")
		qsort swatches sortFn
	),

	fn sortByHSV =
	(
		fn sortFn v1 v2 = 
		(
			local a1 = #(v1.color.h, v1.color.s, v1.color.v)
			local a2 = #(v2.color.h, v2.color.s, v2.color.v)
			arrayCompare a1 a2
		)
		qsort swatches sortFn
	),


	fn sortByLuminance =
	(
		fn sortFn v1 v2 = 
		(
			local l1 = (s_color v1.color).toLum()
			local l2 = (s_color v2.color).toLum()
			case of (
				(l1 == l2): 0
				(l1 > l2): 1
				(l1 < l2): -1
			)
		)
		qsort swatches sortFn
	),
	
	-- Search functions
	fn searchByName str =
	(
		local s_result
		struct s_result (index, match)

		local sortFn		
		fn sortFn v1 v2 = 
		(
			if v1.match < v2.match then 1 
			else if v1.match > v2.match then -1 
			else if v1.index > v2.index then 1
			else if v1.index < v2.index then -1
			else 0
		)

		local pat = "*" + str + "*"
		local n = str.count as float
		local results = for i = 1 to swatches.count where (matchPattern swatches[i].name pattern:pat ignoreCase:true) collect s_result i ((n / swatches[i].name.count) * 100.)
		qsort results sortFn
		results
	),
	
	fn searchByColor col likeness:0.7 =
	(
		local s_result
		struct s_result (index, match)

		local sortFn		
		fn sortFn v1 v2 = 
		(
			if v1.match < v2.match then 1 
			else if v1.match > v2.match then -1 
			else if v1.index > v2.index then 1
			else if v1.index < v2.index then -1
			else 0
		)
		local results = #()
		for i = 1 to swatches.count do (
			local diff = distance [col.r, col.g, col.b] [swatches[i].color.r, swatches[i].color.g, swatches[i].color.b]
			diff /= length [255,255,255]
			diff = 1 - diff
			if diff >= likeness then
				append results (s_result i diff)
		)
		qsort results sortFn
		results
	)

	
) -- end of s_palette struct

