--******************************************************************************
--*                                                                            *
--*  APC (Alternate Color Picker) v1.1 for Max 8                               *
--*  by Ofer Zelichover (c) 12/2004                                            *
--*  Last update: 4:28 PM Tuesday, June 12, 2007
--*  http://www.oferz.com  ; maxscripts@oferz.com                              *
--*                                                                            *
--******************************************************************************
--*  You may use this script freely as you see fit.                            *
--*  You may use parts or the script as a whole in your own scripts.           *
--*  (it would be nice if you give me a creadit if you do so ;))               *
--******************************************************************************
--*  This script comes with no warranty.                                       *
--*  I cannot be held responsible for any loss of data, time or any other      *
--*  damage or loss due to the use of this utility.                            *
--*                                                                            *
--*  ********************************************************************      *
--*  *** IF YOU DON'T LIKE THE ABOVE STATEMENT, DON'T USE THIS SCRIPT ***      *
--*  ********************************************************************      *
--*                                                                            *
--*  **** This script was written for max 7 and wasn't tested on other   ****  *
--*  **** vertions of max.                                               ****  *
--*                                                                            *
--*  If you find any bugs in it please let me know.                            *
--******************************************************************************
--* Description                                                                *
--* -------------                                                              *
--*  This is a photoshop style color picker.                                   *
--*  Please see known isssues below.                                           *
--*                                                                            *
--******************************************************************************
--* History, Status and Known issues                                           *
--* ----------------------------------                                         *
--* Created : 10/9/2001                                                        *
--* update  : v0.95 20/9/2001 - seem to have managed to rid of most bugs,      *
--*             ready for first public beta apperance.                         *
--* update :  v0.96 29/9/2001 - it now saves the needed maps in                *
--*           3dsmax\scripts\ACP for faster loading. it generates the missing  *
--*           maps, so if theres a problem with any map, just delete it.       *
--* update :  v0.97 8/11/2001 - added Lab support                              *
--* update :  v0.98 6/12/2004                                                  *
--*            - removed Lab support. I'm not sure the calculations were right *
--*            - added CMYK support.                                           *
--*            - changed the script to use the s_Color struct. now all color   *
--*              conversions are done through the struct.                      *
--*            - added a currentColor variable (s_Color struct) that holds the *
--*              current color, and is used to do all the conversions.         *
--*            - added a "Restore original color" function.                    *
--* update :  v1.0 17/12/2004                                                  *
--*            - Added integration with color palettes                          
--*            - Created a mzp package for easy installation.
--*                                                                            *
--* Known issues:                                                              *
--*  I know it's not the fastest, I made it mainly to see if it's posible, and *
--*  to try some bitmap manipulations. Although it's not as fast as I would    *
--*  like it to be, I find it can be still be used.                            *
--*  I still want to try to optimize it some more to improve speed.            *
--*  Although it take quite some time when you first open it, the              *
--*  next times are not as bad, since v0.96 it saves the maps.                 *
--*                                                                            *
--*                                                                            *
--*  Plans for the future:                                                     *
--*  Want to improve speed (don't know if i have the knowledge to do that)     *
--*  Add some other types of color pickers.                                    *
--*                                                                            *
--*                                                                            *
--******************************************************************************
--* Isntallation:                                                              *
--* --------------                                                             *
--*    put:                                                                    *
--*            acp0_96.ms   - anywhere                                         *
--*                                                                            *
--*   after you run the script you will find it in the customize user interface*
--*   under "Os Tools" category.                                               *
--*                                                                            *
--******************************************************************************


macroScript ACP

	category:"Os Tools"
	tooltip:"Alternate Color Picker v1.1"
	buttontext:"ACP"
(

	global ro_ACPAboutRollout
	global ro_ACPPhotoshop


	try (destroyDialog ro_ACPPhotoshop) catch()


	rollout ro_ACPAboutRollout "About ACP 1.0"
	(
		label psAboutLbl_01 "ACP (Alternate Color Picker) 1.0"
		label psAboutLbl_02 "By Ofer Zelichover (c) 12/2004"
		label psAboutLbl_03 ""
		label psAboutLbl_04 "For more details about this script, please see the script file."
		label psAboutLbl_05 ""
		label psAboutLbl_06 "Usage: Set the color and drag the color in the right top square to the max color field."
		label psAboutLbl_07 ""
		label psAboutLbl_08 "This script comes with no warranty."
		label psAboutLbl_09 "I cannot be held responsible for any loss of data, time or any other"
		label psAboutLbl_10 "damage or loss due to the use of this utility."
		label psAboutLbl_11 ""
		button psAboutWinClose "Close"
		
		on psAboutWinClose pressed do (gc light:true; destroyDialog ro_ACPAboutRollout)
	) -- End About rollout
	
	
	
	rollout ro_ACPPhotoshop "ACP 1.0"
	(
	-- Struct Definitions
	--====================
		include "ACP\\Structs\\rect_struct.ms"
		include "ACP\\Structs\\color_struct.ms"



	-- Global Variable Delerations
	--=============================
		global ro_ACPPalette
	
		global ACPClipBoard
		global psColorByHue
		global psColorBySat
		global psColorByVal
		global psColorByR
		global psColorByG
		global psColorByB
		global psColorByB
	
	-- Local Variable Delerations
	--=============================
		-- Map variables
		local psCurrentColorMap	-- current color selection map
		local psCurrentColorBar	-- current color parameter bar
		local psColorBarByHue
		local psColorBarBySat
		local psColorBarByVal
		local psColorBarByR
		local psColorBarByG
		local psColorBarByB
		local psSliderMap
	
	
		-- Other variables
		local currentColor = s_color() -- this variable holds the current color. all color conversions are made through this struct.
		local mouseDrag = false		-- determines if the in drag mode
		local mouseDragIn = 0		-- which part of the dialog to drag (0-rest, 1-colorbitmap, 2-colorslider, 3-colorhuebitmap)
		local psColorMode = 1		-- 1-By HUE; 2-By Sat; 3-By Val; 4-By R; 5-By G; 6-By B;
		local psOrigColor = if not (isKindOf ACPClipBoard Color) then red else ACPClipBoard
		local prevPos, prevSliderVal, prevHexVal, prevPickerValue
	
		-- marker shape array	
		local markerShapeArr = #([-2,-5],[-1,-5],[0,-5],[1,-5],[2,-5], \
						 [-3,-4],[3,-4], \
						 [-4,-3],[4,-3], \
						 [-5,-2],[5,-2], \
						 [-5,-1],[5,-1], \
						 [-5,-0],[5,-0], \
						 [-5,1],[5,1],   \
						 [-5,2],[5,2],   \
						 [-4,3],[4,3],   \
						 [-3,4],[3,4],   \
						 [-2,5],[-1,5],[0,5],[1,5],[2,5])
	
	
		
	-- Local Function Declerations
	--=============================
		local genColorMapByHue
		local genColorBarByHue

		local updatePalette
		local isPalettesRolloutOpen
		
		
	-- GUI
	--========
	-- GUI Bitmaps
		groupbox psColorGrpColor "" width:468 height:272 pos:[1,-4]
		bitmap psColorBitmap bitmap:(bitmap 260 256) pos:[5,5]
		bitmap psColorSlider bitmap:(bitmap 10 256) pos:[272,5]
		bitmap psColorBar bitmap:(bitmap 20 256) pos:[280,5]
		button psOrigColorButton images:#((bitmap 65 60 color:psOrigColor), undefined, 1, 1, 1, 1, 1) pos:[318,10] height:60 width:65 tooltip:"Original color. Click to restore."
		colorpicker psCurrentColor color:psOrigColor fieldwidth:65 height:60 pos:[380,10]
	
	-- mode selector labels
		label psLbl_01 "H:" width:17 pos:[320,78]
		label psLbl_02 "S:" width:17 pos:[320,103]
		label psLbl_03 "V:" width:17 pos:[320,128]
		label psLbl_04 "R:" width:17 pos:[320,163]
		label psLbl_05 "G:" width:17 pos:[320,188]
		label psLbl_06 "B:" width:17 pos:[320,213]
		label psLbl_10 "C:" width:17 pos:[397,78]
		label psLbl_11 "M:" width:17 pos:[397,103]
		label psLbl_12 "Y:" width:17 pos:[397,128]
		label psLbl_13 "K:" width:17 pos:[397,153]
	-- mode selectors
		checkbutton psHueMode "" width:12 height:12 pos:[305,79] checked:true highlightcolor:(red/2)
		checkbutton psSatMode "" width:12 height:12 pos:[305,104] checked:false highlightcolor:(red/2)
		checkbutton psValMode "" width:12 height:12 pos:[305,129] checked:false highlightcolor:(red/2)
		checkbutton psRMode "" width:12 height:12 pos:[305,164] checked:false highlightcolor:(red/2)
		checkbutton psGMode "" width:12 height:12 pos:[305,189] checked:false highlightcolor:(red/2)
		checkbutton psBMode "" width:12 height:12 pos:[305,214] checked:false highlightcolor:(red/2)
	-- values fields
		spinner psHue "" range:[0,360,0] type:#integer fieldWidth:32 pos:[332,77]
		label psLbl_101 "" pos:[377,77]
		spinner psSat "" range:[0,100,100] type:#integer fieldWidth:32 pos:[332,102]
		label psLbl_102 "%" pos:[377,103]
		spinner psVal "" range:[0,100,100] type:#integer fieldWidth:32 pos:[332,127]
		label psLbl_103 "%" pos:[377,128]
		spinner psR "" range:[0,255,255] type:#integer fieldWidth:32 pos:[332,162]
		spinner psG "" range:[0,255,0] type:#integer fieldWidth:32 pos:[332,187]
		spinner psB "" range:[0,255,0] type:#integer fieldWidth:32 pos:[332,212]
		spinner psCMYK_C "" range:[0,100,0] type:#integer fieldWidth:32 pos:[407,77]
		label psLbl_104 "%" pos:[452,78]
		spinner psCMYK_M "" range:[0,100,0] type:#integer fieldWidth:32 pos:[407,102]
		label psLbl_105 "%" pos:[452,103]
		spinner psCMYK_Y "" range:[0,100,0] type:#integer fieldWidth:32 pos:[407,127]
		label psLbl_106 "%" pos:[452,128]
		spinner psCMYK_K "" range:[0,100,0] type:#integer fieldWidth:32 pos:[407,152]
		label psLbl_107 "%" pos:[452,153]
		edittext psHexValue "#: " fieldWidth:50 pos:[307,245]
		
	-- general buttons	
		button psAbout "About" pos:[170,275] height:21 width:65
		checkbutton psPalette "Palettes..." pos:[250,275] height:21 width:65 checked:(isPalettesRolloutOpen())
		button psCancel "Cancel" pos:[330,275] height:21 width:65
		button psClose "Ok" pos:[400,275] height:21 width:65
		
	-- test
--		groupbox cpTypeGrpPS "" width:160 height:22 pos:[5,273]
--			label cpTypeTextPS "Photoshop Style Color Picker" pos:[17,280]
			
	-- timers
		timer psColorMapUpdateTimer interval:500 active:false
	
	
	-- Functions
	--===========
	-- General Functions
	--------------------
		fn isPalettesRolloutOpen = (try(ro_ACPPalette.open)catch(false))
		-- Checks for file existance
		fn existFile fname = ((getFiles fname).count!=0)
		-- Checks for directory existance
		fn existDir dname = ((getDirectories dname).count!=0)
	
		-- takes a number and returns as string with leading zeros
		fn leadingZero num len =
		(
			local t = num as string
			while t.count < len do 
				t = "0" + t
			t
		)
	
	
	
		-- returns the y min and max values from a point2 array
		fn arrYMinMax arr &ymin &ymax = 
		(
			local a = #()
			try (
				a = for i in arr collect i.y
			) catch (ymin=undefined;ymax=undefined)
			ymin = amin a
			ymax = amax a
			OK
		)
	
		-- returns the x min and max values from a point2 array
		fn arrXMinMax arr &xmin &xmax = 
		(
			local a = #()
			try (
				a = for i in arr collect i.x
			) catch (xmin=undefined;xmax=undefined)
			xmin = amin a
			xmax = amax a
			OK
		)
	
	--GUI related functions
	-----------------------
		-- Validate hex input
		fn checkHexVal hexVal = 
		(
			local hexChars = "0123456789ABCDEFabcdef"
			if hexVal == undefined or hexVal == "" then
				return ""
			for i = 1 to hexVal.count do (
				if (findString hexChars hexVal[i]) == unefined then
					return prevHexVal
			)
			hexVal
		)
		
		-- Draw the marker on the color map
		fn drawMarker origBitmap oldPos pos &b = 
		(
			arrYMinMax markerShapeArr &ymin &ymax
			arrXMinMax markerShapeArr &xmin &xmax
			for y = ymin to ymax do
				for x = xmin to xmax do
					setPixels b (point2 (oldPos.x+x) (oldPos.y+y)) (getPixels origBitmap (point2 (oldPos.x+x) (oldPos.y+y)) 1)
			local col =  if ((getPixels b pos 1)[1]).v < (255*0.6) then white else black
			for i = 1 to markerShapeArr.count do
				setPixels b (pos+markerShapeArr[i]) #(col)
			prevPos = pos
		)
	
	-- Mouse Related Functions

		
	-- Map generation functions
	-----------------------------
--	include "ACP\\generate_maps.ms"
		-- generate color bar for hue mode
		fn genColorBarByHue = (
			local bm = bitmap 20 256
			local col = color 255 0 0
			for y = 0 to 255 do (
				local colArr = #()
				col.h = 255.-y
				for x = 1 to 20 do (
					colArr[x] = col
				)
				setPixels bm (point2 0 y) colArr
			)
			gc light:true
			bm
		)
	
		-- generate color bar from col to black for sat,val,r,g,b modes
		fn genColorBarByVal col = (
			local bm = bitmap 20 256
			for y = 0 to 255 do (
				local colArr = #()
				col.v = 255.-y
				for x = 1 to 20 do (
					colArr[x] = col
				)
				setPixels bm (point2 0 y) colArr
			)
			gc light:true
			bm
		)
	
		-- generate color map for val mode
		fn genColorMapByVal = (
			local x,y
			local bm = bitmap 256 256
			local colArr
			local c = s_color()
			for y = 0 to 255 do (
				colArr = #()
				for x = 0 to 255 do (
					c.fromHSV x (255-y) 255
					colArr[x+1] = c.colorValue
				)
				setPixels bm (point2 0 y) colArr
			)
			gc ()
			bm
		)
	
	
		-- generate color map for sat mode
		fn genColorMapBySat = (
			local x,y
			local bm = bitmap 256 256
			local colArr = #()
			local c = s_color()
			for y = 0 to 255 do (
				colArr = #()
				for x = 0 to 255 do (
					c.fromHSV x 255 (255-y)
					colArr[x+1] = c.colorValue
				)
				setPixels bm (point2 0 y) colArr
			)
			gc ()
			bm
		)
	
		-- generate color map for hue mode	
		fn genColorMapByHue = (
			local x,y
			local bm = bitmap 256 256
			local colArr = #()
			local c = s_color()
			for y = 0 to 255 do (
				colArr = #()
				for x = 0 to 255 do (
					c.fromHSV 0 x (255-y) ignorePrevious:true
					colArr[x+1] = c.colorValue
				)
				setPixels bm (point2 0 y) colArr
			)
			gc ()
			bm
		)
	
		-- generate color map for R mode	
		fn genColorMapByR = (
			local x,y
			local bm = bitmap 256 256
			for y = 0 to 255 do (
				local colArr = #()
				for x = 0 to 255 do (
					colArr[x+1] = (color 0 (255-y) x)
				)
				setPixels bm (point2 0 y) colArr
			)
			gc ()
			bm
		)
	
		-- generate color map for G mode
		fn genColorMapByG = (
			local x,y
			local bm = bitmap 256 256
			for y = 0 to 255 do (
				local colArr = #()
				for x = 0 to 255 do (
					colArr[x+1] = (color (255-y) 0 x)
				)
				setPixels bm (point2 0 y) colArr
			)
			gc ()
			bm
		)
	
		-- generate color map for B mode
		fn genColorMapByB = (
			local x,y
			local bm = bitmap 256 256
			for y = 0 to 255 do (
				local colArr = #()
				for x = 0 to 255 do (
					colArr[x+1] = (color x (255-y) 0)
				)
				setPixels bm (point2 0 y) colArr
			)
			gc ()
			bm
		)
	
	
		
	--update maps
	--------------
		-- update color map by hue
		fn updateColorMapByHue &b hue = 
		(
			for y = 0 to 255 do (
				local colArr = (getPixels b (point2 0 y) 256)
				for x = 1 to 256 do (
					colArr[x].h = hue
				)
				setPixels b (point2 0 y) colArr
			)
		)
	
		-- update color map by sat
		fn updateColorMapBySat &b sat = 
		(
			for y = 0 to 255 do (
				local colArr = (getPixels b (point2 0 y) 256)
				for x = 1 to 256 do (
					colArr[x].s = sat
				)
				setPixels b (point2 0 y) colArr
			)
		)
	
		-- update color map by val
		fn updateColorMapByVal &b val = 
		(
			for y = 0 to 255 do (
				local colArr = (getPixels b (point2 0 y) 256)
				for x = 1 to 256 do (
					colArr[x].v = val
				)
				setPixels b (point2 0 y) colArr
			)
		)
	
		-- update color map by RGB values
		fn updateColorMapByRGB &b col = 
		(
			for y = 0 to 255 do (
				local colArr = (getPixels b (point2 0 y) 256)
				for x = 1 to 256 do (
					colArr[x] += col
				)
				setPixels b (point2 0 y) colArr
			)
		)
	
		-- updater slider map
		fn updateSliderMap oldPos pos &b = 
		(
			local colArr = #()
			for i = 1 to b.width do
				colArr[i] = black
			setPixels b (point2 0 (b.height-oldPos-1)) colArr
			for i = 1 to b.width do
				colArr[i] = white
			setPixels b (point2 0 (b.height-pos-1)) colArr
		)
	
	-- update other GUI
	--------------------
		-- updates the CurrentColor's color by RGB values
		fn updateCurrentColor = 
		(
			psCurrentColor.color = currentColor.colorValue
		)
	
		-- update the psHexValue's value by the RGB values
		fn updateHexVal = 
		(
			local h = currentColor.toHex()
			prevHexVal = h
			psHexValue.text = h
		)
	
		-- Update the RGB GUI values using col.
		fn updateRGBVals = 
		(
			currentColor.toRGB &psR.value &psG.value &psB.value
		)
	
		-- Update the HSV GUI values using CurrentColor's color.
		fn updateHSVvals = 
		(
			local h, s, v
			currentColor.toHSV &h &s &v
			psHue.value = 360*(h/255.)
			psSat.value = 100*(s/255.)
			psVal.value = 100*(v/255.)
		)
	

		fn updateCMYKVals = 
		(
			local C, M, Y, K
			currentColor.toCMYK &C &M &Y &K
			psCMYK_C.value = C * 100
			psCMYK_M.value = M * 100
			psCMYK_Y.value = Y * 100
			psCMYK_K.value = K * 100
		)

	
	fn updateAllFields withPalette:true =
		(
			updateCurrentColor()
			ACPClipBoard = currentColor.colorValue
			updateRGBVals()
			updateHSVvals()
			updateHexVal()
			updateCMYKVals()
			-- update the palette window, if it's open.
			if withPalette then
				updatePalette()
		)
	

		-- update RGB and HSV fields by the psHexValue field
		fn updateRGBbyHex = (
			currentColor.fromHex psHexValue.text
			updateAllFields()
		)


	-- Update current maps
	------------------------
		-- redraw color map using psCurrentColorMap , and draw marker according to mode
		fn redrawColorMap = (
			case psColorMode of (
				1:	pos = point2 currentColor.colorValue.s (255 - currentColor.colorValue.v)
				2:	pos = point2 currentColor.colorValue.h (255 - currentColor.colorValue.v)

				3:	pos = point2 currentColor.colorValue.s (255 - currentColor.colorValue.s)
				4:	pos = point2 currentColor.colorValue.b (255 - currentColor.colorValue.g)
				5:	pos = point2 currentColor.colorValue.b (255 - currentColor.colorValue.r)
				6:	pos = point2 currentColor.colorValue.r (255 - currentColor.colorValue.g)

			)
			psColorBitmap.bitmap = copy psCurrentColorMap
			drawMarker psCurrentColorMap prevPos pos &psColorBitmap.bitmap
			psColorBitmap.bitmap = psColorBitmap.bitmap
		)
	
		-- updates psCurrentColorMap according to mode
		fn updateCurrentColorMap = (
			case psColorMode of (
				1:	(
						psCurrentColorMap = copy psColorByHue
						updateColorMapByHue &psCurrentColorMap currentColor.colorValue.h
					)
				2:	(
						psCurrentColorMap = copy psColorBySat
						updateColorMapBySat &psCurrentColorMap currentColor.colorValue.s
					)
				3:	(
						psCurrentColorMap = copy psColorByVal
						updateColorMapByVal &psCurrentColorMap currentColor.colorValue.v
					)
				4:	(
						psCurrentColorMap = copy psColorByR
						updateColorMapByRGB &psCurrentColorMap (color currentColor.colorValue.r 0 0)
					)
				5:	(
						psCurrentColorMap = copy psColorByG
						updateColorMapByRGB &psCurrentColorMap (color 0 currentColor.colorValue.g 0)
					)
				6:	(
						psCurrentColorMap = copy psColorByB
						updateColorMapByRGB &psCurrentColorMap (color 0 0 currentColor.colorValue.b)
					)
			)
			redrawColorMap()
			gc light:true
		)
	
		-- update field values from colorMap pos according to mode
		-- and draw the marker in the right place.
		fn changeValsByColorMapPos pos = (
			case psColorMode of (
				1:	(
						currentColor.fromHSV currentColor.colorValue.h pos.x (255-pos.y)
						updateAllFields()
					)
				2:	(
						currentColor.fromHSV pos.x currentColor.colorValue.s (255-pos.y)
						updateAllFields()
					)
				3:	(
						currentColor.fromHSV pos.x (255-pos.y) currentColor.colorValue.v
						updateAllFields()
					)
				4:	(
						currentColor.fromRGB currentColor.colorValue.r (255-pos.y) pos.x
						updateAllFields()
					)
				5:	(
						currentColor.fromRGB (255-pos.y) currentColor.colorValue.g pos.x
						updateAllFields()
					)
				6:	(
						currentColor.fromRGB pos.x (255-pos.y) currentColor.colorValue.b
						updateAllFields()
					)
			)
			drawMarker psCurrentColorMap prevPos pos &psColorBitmap.bitmap
			psColorBitmap.bitmap = psColorBitmap.bitmap
		)
	
		-- updates the slider map according to the color mode and color
		fn updateSlider = (
			local val
			case psColorMode of (
				1:	val = currentColor.colorValue.h
				2:	val = currentColor.colorValue.s
				3:	val = currentColor.colorValue.v
				4:	val = currentColor.colorValue.r
				5:	val = currentColor.colorValue.g
				6:	val = currentColor.colorValue.b
			)
			updateSliderMap prevSliderVal val &psColorSlider.bitmap
			psColorSlider.bitmap = psColorSlider.bitmap
			prevSliderVal = val
		)
	
		-- update field values by barMap pos according to mode
		fn changeValsByBarMapPos pos = (
			case psColorMode of (
				1:	currentColor.fromHSV (255-pos.y) currentColor.colorValue.s currentColor.colorValue.v
				2:	currentColor.fromHSV currentColor.colorValue.h (255-pos.y) currentColor.colorValue.v
				3:	currentColor.fromHSV currentColor.colorValue.h currentColor.colorValue.s (255-pos.y)
				4:	currentColor.fromRGB (255-pos.y) currentColor.colorValue.g currentColor.colorValue.b
				5:	currentColor.fromRGB currentColor.colorValue.r (255-pos.y) currentColor.colorValue.b
				6:	currentColor.fromRGB currentColor.colorValue.r currentColor.colorValue.g (255-pos.y)
			)
			updateAllFields()
			updateSlider()
		)
	
		-- checks if the mouse is in one of the maps, and execute actions accordingly
		fn checkMouseMode pos = (
			--	mouseDrag  - determines if the in drag mode
			--	mouseDragIn - which part of the dialog to drag (0-rest, 1-colorbitmap, 2-colorslider, 3-colorhuebitmap)
			local colorMapRect = s_rect (psColorBitmap.pos) ((psColorBitmap.pos+[psColorBitmap.bitmap.width,psColorBitmap.bitmap.height])-[1,1])
			local colorSliderMapRect = s_rect (psColorSlider.pos) ((psColorSlider.pos+[psColorSlider.bitmap.width,psColorSlider.bitmap.height])-[1,1])
			local colorBarRect = s_rect (psColorBar.pos) ((psColorBar.pos+[psColorBar.bitmap.width,psColorBar.bitmap.height])-[1,1])
			
			if ((colorMapRect.isInside pos and NOT mouseDrag)or (mouseDrag and mouseDragIn==1)) then (
				mouseDrag = true
				mouseDragIn = 1
				local p = (colorMapRect.keepInside pos) - colorMapRect.ul
				changeValsByColorMapPos p
			)
			if ((colorSliderMapRect.isInside pos and NOT mouseDrag) or (mouseDrag and mouseDragIn==2)) then (
				mouseDrag = true
				mouseDragIn = 2
				local p = (colorSliderMapRect.keepInside pos) - colorSliderMapRect.ul
				changeValsByBarMapPos p
			)
			if ((colorBarRect.isInside pos and NOT mouseDrag) or (mouseDrag and mouseDragIn==3)) then (
				mouseDrag = true
				mouseDragIn = 3
				local p = (colorBarRect.keepInside pos) - colorBarRect.ul
				changeValsByBarMapPos p
			)
		)
	
		-- change color mode
		fn changeColorMode = (
			case psColorMode of (
				1:	(
						updateCurrentColorMap()
						psCurrentColorBar = psColorBarByHue
					)
				2:	(
						updateCurrentColorMap()
						psCurrentColorBar = psColorBarBySat
					)
				3:	(
						updateCurrentColorMap()
						psCurrentColorBar = psColorBarByVal
					)
				4:	(
						updateCurrentColorMap()
						psCurrentColorBar = psColorBarByR
					)
				5:	(
						updateCurrentColorMap()
						psCurrentColorBar = psColorBarByG
					)
				6:	(
						updateCurrentColorMap()

						psCurrentColorBar = psColorBarByB
					)
			)
			psColorBar.bitmap = copy psCurrentColorBar
			redrawColorMap()
			updateSlider()
		)
	
		-- turns mode selectors on/off according to mode
		fn updateModeButtons n = (
			psHueMode.checked = n==1
			psSatMode.checked = n==2
			psValMode.checked = n==3
			psRMode.checked = n==4
			psGMode.checked = n==5
			psBMode.checked = n==6
			psColorMode = n
			changeColorMode()
		)
	
	-- Init functions
		fn createMaps forceGenerate:false = (
			local psCreateMaps
			local prevPNGType = Portable_Network_Graphics.ipngio.getType()
			
			Portable_Network_Graphics.ipngio.setType #true24
			

			-- Create a message winodw saying maps are being created...
			rollout psCreateMaps "Creating Maps..."
			(
				label lbl "Creating maps for color picker, please wait..." align:#center offset:[0,15]
			)
			createDialog psCreateMaps 300 50 style:#(#style_toolwindow)
	
			-- Check if ACP folder exists, if not create it.
			if NOT (existDir (getDir #scripts + "\\ACP")) then
				makeDir (getDir #scripts + "\\ACP")
			
			local ACPDir = (getDir #scripts + "\\ACP\\")
			
			-- Check if maps are already in memory, if not check if they are in ACP folder,
			-- if so load them, if not, create them and save to ACP folder.
			if psColorByHue == undefined or forceGenerate then (
				if existFile (ACPDir + "psColorByHue.png") and not forceGenerate then (
					psColorByHue = openBitmap (ACPDir + "psColorByHue.png")
				) else (
					psColorByHue = genColorMapByHue()
					psColorByHue.filename = (ACPDir + "psColorByHue.png")
					save psColorByHue
				)
			)
			if psColorBySat == undefined or forceGenerate then (
				if existFile (ACPDir + "psColorBySat.png") and not forceGenerate then (
					psColorBySat = openBitmap (ACPDir + "psColorBySat.png")
				) else (
					psColorBySat = genColorMapBySat()
					psColorBySat.filename = (ACPDir + "psColorBySat.png")
					save psColorBySat
				)
			)
			if psColorByVal == undefined or forceGenerate then (
				if existFile (ACPDir + "psColorByVal.png") and not forceGenerate then (
					psColorByVal = openBitmap (ACPDir + "psColorByVal.png")
				) else (
					psColorByVal = genColorMapByVal()
					psColorByVal.filename = (ACPDir + "psColorByVal.png")
					save psColorByVal
				)
			)
			if psColorByR == undefined or forceGenerate then (
				if existFile (ACPDir + "psColorByR.png") and not forceGenerate then (
					psColorByR = openBitmap (ACPDir + "psColorByR.png")
				) else (
					psColorByR = genColorMapByR()
					psColorByR.filename = (ACPDir + "psColorByR.png")
					save psColorByR
				)
			)
			if psColorByG == undefined or forceGenerate then (
				if existFile (ACPDir + "psColorByG.png") and not forceGenerate then (
					psColorByG = openBitmap (ACPDir + "psColorByG.png")
				) else (
					psColorByG = genColorMapByG()
					psColorByG.filename = (ACPDir + "psColorByG.png")
					save psColorByG
				)
			)
			if psColorByB == undefined or forceGenerate then (
				if existFile (ACPDir + "psColorByB.png") and not forceGenerate then (
					psColorByB = openBitmap (ACPDir + "psColorByB.png")
				) else (
					psColorByB = genColorMapByB()
					psColorByB.filename = (ACPDir + "psColorByB.png")
					save psColorByB
				)
			)
	

			Portable_Network_Graphics.ipngio.setType prevPNGType
	
			-- Create the bar maps
			psColorBarByHue = genColorBarByHue()
			psColorBarBySat = genColorBarByVal white
			psColorBarByVal = genColorBarByVal white
			psColorBarByR = genColorBarByVal red
			psColorBarByG = genColorBarByVal green
			psColorBarByB = genColorBarByVal blue
	
			-- Close the message window...
			destroyDialog psCreateMaps

		)
	


		-- Integration with the color palette
		fn updateFromPalette = 
		(
			currentColor.colorValue = ACPClipBoard
			psOrigColor = ACPClipBoard
			psOrigColorButton.images = #((bitmap 65 60 color:psOrigColor), undefined, 1, 1, 1, 1, 1)
			updateAllFields withPalette:false
			updateSlider()
			updateCurrentColorMap()
		)
	
		fn updatePalette = 
		(
			try (
				ro_ACPPalette.updateFromColorPicker()
			)catch()
		)
	
		-- create the about window
		fn psAboutWindow = (
			createDialog ro_ACPAboutRollout 420 230 style:#(#style_titlebar, #style_border)
		)
	
		-- initialize some variables
		fn init = (
			currentColor.colorValue = psOrigColor
			psOrigColorButton.images = #((bitmap 65 60 color:psOrigColor), undefined, 1, 1, 1, 1, 1)
			psCurrentColor.color = psOrigColor

			prevPos = [currentColor.colorValue.s,currentColor.colorValue.v]
			prevSliderVal = currentColor.colorValue.h
			
			createMaps()
			changeColorMode()
	
			psSliderMap = bitmap 10 256 color:(color 0 0 0)
			psColorSlider.bitmap = psSliderMap
			updateSlider()
			updateCurrentColorMap()
			updateAllFields()
		)
	
	-- Event Handlers
	--================
	-- Color picker changed
		on psCurrentColor changed col do (
			currentColor.colorValue = col
			if col != prevPickerValue then (
				psColorMapUpdateTimer.active = false
				psColorMapUpdateTimer.active = true
			)
			updateAllFields()
			updateSlider()
			prevPickerValue = col
		)
		
		on psColorMapUpdateTimer tick do (
			updateCurrentColorMap()
			psColorMapUpdateTimer.active = false
		)
	

	-- Restore original color
		on psOrigColorButton pressed do (
			currentColor.colorValue = psOrigColor
			updateAllFields()
			updateSlider()
			updateCurrentColorMap()
		)

	-- HSV values change
		on psHue changed value do (
			value = 255.*(value/360.)
			currentColor.fromHSV value currentColor.colorValue.s currentColor.colorValue.v
			case psColorMode of (
				1:	updateSlider()
				2:	(
						drawMarker psCurrentColorMap prevPos [value,(255-currentColor.colorValue.v)] &psColorBitmap.bitmap
						psColorBitmap.bitmap = psColorBitmap.bitmap
					)
				3:	(
						drawMarker psCurrentColorMap prevPos [value,(255-currentColor.colorValue.s)] &psColorBitmap.bitmap
						psColorBitmap.bitmap = psColorBitmap.bitmap
					)
			)
			updateAllFields()
		)
		on psHue buttonup do (if psColorMode == 1 then updateCurrentColorMap())
		on psSat changed value do (
			value = 255.*(value/100.)
			currentColor.fromHSV currentColor.colorValue.h value currentColor.colorValue.v
			case psColorMode of (
				1:	(
						drawMarker psCurrentColorMap prevPos [value,(255-currentColor.colorValue.v)] &psColorBitmap.bitmap
						psColorBitmap.bitmap = psColorBitmap.bitmap
					)
				2:	updateSlider()
				3:	(
						drawMarker psCurrentColorMap prevPos [currentColor.colorValue.h,(255-value)] &psColorBitmap.bitmap
						psColorBitmap.bitmap = psColorBitmap.bitmap
					)
			)
			updateAllFields()
		)
		on psSat buttonup do (if psColorMode == 2 then updateCurrentColorMap())
		on psVal changed value do (
			value = 255.*(value/100.)
			currentColor.fromHSV currentColor.colorValue.h currentColor.colorValue.s value
			case psColorMode of (
				1:	(
						drawMarker psCurrentColorMap prevPos [currentColor.colorValue.s,(255-value)] &psColorBitmap.bitmap
						psColorBitmap.bitmap = psColorBitmap.bitmap
					)
				2:	(
						drawMarker psCurrentColorMap prevPos [currentColor.colorValue.h,(255-value)] &psColorBitmap.bitmap
						psColorBitmap.bitmap = psColorBitmap.bitmap
					)
				3:	updateSlider()
			)
			updateAllFields()
		)
		on psVal buttonup do (if psColorMode == 3 then updateCurrentColorMap())
	
	-- RGB values change
		on psR changed value do (
			currentColor.colorValue.r = value
			case psColorMode of (
				4:	updateSlider()
				5:	(
						drawMarker psCurrentColorMap prevPos [currentColor.colorValue.b,(255-value)] &psColorBitmap.bitmap
						psColorBitmap.bitmap = psColorBitmap.bitmap
					)
				6:	(
						drawMarker psCurrentColorMap prevPos [value,(255-currentColor.colorValue.g)] &psColorBitmap.bitmap
						psColorBitmap.bitmap = psColorBitmap.bitmap
					)
			)
			updateAllFields()
		)
		on psR entered do (updateCurrentColorMap())
		on psR buttonup do (updateCurrentColorMap())
	
		on psG changed value do (
			currentColor.colorValue.g = value
			case psColorMode of (
				4:	(
						drawMarker psCurrentColorMap prevPos [currentColor.colorValue.b,(255-value)] &psColorBitmap.bitmap
						psColorBitmap.bitmap = psColorBitmap.bitmap
					)
				5:	updateSlider()
				6:	(
						drawMarker psCurrentColorMap prevPos [currentColor.colorValue.r,(255-value)] &psColorBitmap.bitmap
						psColorBitmap.bitmap = psColorBitmap.bitmap
					)
			)
			updateAllFields()
		)
		on psG entered do (updateCurrentColorMap())
		on psG buttonup do (updateCurrentColorMap())
	
		on psB changed value do (
			currentColor.colorValue.b = value
			case psColorMode of (
				4:	(
						drawMarker psCurrentColorMap prevPos [value,(255-currentColor.colorValue.g)] &psColorBitmap.bitmap
						psColorBitmap.bitmap = psColorBitmap.bitmap
					)
				5:	(
						drawMarker psCurrentColorMap prevPos [value,(255-currentColor.colorValue.r)] &psColorBitmap.bitmap
						psColorBitmap.bitmap = psColorBitmap.bitmap
					)
				6:	updateSlider()
			)
			updateAllFields()
		)
		on psB entered do (updateCurrentColorMap())
		on psB buttonup do (updateCurrentColorMap())
		
	-- HEX values change
		on psHexValue changed text do (
			if text.count > 6 or (checkHexVal text != text) then (
				psHexValue.text = if prevHexVal != undefined then prevHexVal else ""
				text = prevHexVal
			)
		)
		on psHexValue entered text do (
			if text.count > 6 or (checkHexVal text != text) then (
				psHexValue.text = prevHexVal
				text = prevHexVal
			)
			if text != prevHexVal then (
				prevHexVal = text
				updateRGBbyHex()
				updateCurrentColorMap()
				updateSlider()
			)
		)
		
	-- CMYK values change
		on psCMYK_C changed value do (
			currentColor.fromCMYK (psCMYK_C.value/100.) (psCMYK_M.value/100.) (psCMYK_Y.value/100.) (psCMYK_K.value/100.)
			updateAllFields()
			updateSlider()
		)
		on psCMYK_C buttonup do (updateCurrentColorMap())			
		on psCMYK_M changed value do (
			currentColor.fromCMYK (psCMYK_C.value/100.) (psCMYK_M.value/100.) (psCMYK_Y.value/100.) (psCMYK_K.value/100.)
			updateAllFields()
			updateSlider()
		)
		on psCMYK_M buttonup do (updateCurrentColorMap())			
		on psCMYK_Y changed value do (
			currentColor.fromCMYK (psCMYK_C.value/100.) (psCMYK_M.value/100.) (psCMYK_Y.value/100.) (psCMYK_K.value/100.)
			updateAllFields()
			updateSlider()
		)
		on psCMYK_Y buttonup do (updateCurrentColorMap())			
		on psCMYK_K changed value do (
			currentColor.fromCMYK (psCMYK_C.value/100.) (psCMYK_M.value/100.) (psCMYK_Y.value/100.) (psCMYK_K.value/100.)
			updateAllFields()
			updateSlider()
		)
		on psCMYK_K buttonup do (updateCurrentColorMap())			
		
		
	-- Modes
		on psHueMode changed state do (updateModeButtons 1)
		on psSatMode changed state do (updateModeButtons 2)
		on psValMode changed state do (updateModeButtons 3)
		on psRMode changed state do (updateModeButtons 4)
		on psGMode changed state do (updateModeButtons 5)
		on psBMode changed state do (updateModeButtons 6)
	
	-- Mouse handlers
		on ro_ACPPhotoshop lbuttondown pos do (checkMouseMode (pos-[2,2]))
		on ro_ACPPhotoshop mousemove pos do (if mouseDrag then checkMouseMode (pos-[2,2]))
		on ro_ACPPhotoshop lbuttonup pos do (
			mouseDrag = false
			case mouseDragIn of (
				2:	updateCurrentColorMap()
				3:	updateCurrentColorMap()
			)
			mouseDragIn = 0
			gc light:true
		)

		
	-- General rollout handlers
		on psAbout pressed do psAboutWindow()
		on psPalette changed state do (
			if state then (
				if ro_ACPPalette == undefined or not ro_ACPPalette.open then 
					macros.run "Os Tools" "ColorPalette"
			) else (
				try(destroyDialog ro_ACPPalette)catch()
			)
		)
		on psCancel pressed do (
			ACPClipBoard = psOrigColor
			try(destroyDialog ro_ACPAboutRollout)catch()
			destroyDialog ro_ACPPhotoshop
			gc light:true
		)
		on psClose pressed do (try(destroyDialog ro_ACPAboutRollout)catch(); destroyDialog ro_ACPPhotoshop; gc light:true)	

		on ro_ACPPhotoshop open do (init())
		on ro_ACPPhotoshop close do ro_ACPPhotoshop = undefined
	
	) -- End ro_ACPPhotoshop rollout
	
	
	
	createDialog ro_ACPPhotoshop 470 300 style:#(#style_titlebar, #style_sysmenu, #style_border)

)-- end of macroscript