--******************************************************************************
--*                                                                            *
--*  Color Palette   v1.1 for Max 8
--*  by Ofer Zelichover (c) 12/2004                                            *
--*  Last update: 4:28 PM Tuesday, June 12, 2007
--*  http://www.oferz.com  ; maxscripts@oferz.com                              *
--*                                                                            *
--******************************************************************************
--*  You may use this script freely as you see fit.                            *
--*  You may use parts or the script as a whole in your own scripts.           *
--*  (it would be nice if you give me a creadit if you do so ;))               *
--******************************************************************************
--*  This script comes with no warranty.                                       *
--*  I cannot be held responsible for any loss of data, time or any other      *
--*  damage or loss due to the use of this utility.                            *
--*                                                                            *
--*  ********************************************************************      *
--*  *** IF YOU DON'T LIKE THE ABOVE STATEMENT, DON'T USE THIS SCRIPT ***      *
--*  ********************************************************************      *
--*                                                                            *
--*  **** This script was written for max 7 and wasn't tested on other   ****  *
--*  **** vertions of max.                                               ****  *
--*                                                                            *
--*  If you find any bugs in it please let me know.                            *
--******************************************************************************
--* Description                                                                *
--* -------------                                                              *
--*  This is a photoshop style color picker.                                   *
--*  Please see known isssues below.                                           *
--*                                                                            *
--*  Abode color file format information is base on this page:
--*  http://www.nomodes.com/aco.html by Larry Tesler
--******************************************************************************
--* History, Status and Known issues                                           *
--* ----------------------------------                                         *
--* Created : 10/12/2004                                                       *

--* Updated : 13/12/2004 ver 0.31
--*     - fixed: crash when canceling a save/load palette operation.
--*     - chanded: the scroll bar is wider.
--*     - added: clear palette option to delete all the color swatches.


--* Updated : 14/12/2004 ver 0.4
--*     - added: duplicate color
--*     - added: move color up/down
--*     - added: global palette. the palette now remembers the last state.


--* Updated : 16/12/2004 ver 0.5
--*     - added: sort palette by name, RGB, HSV and luminance
--*     - added: generate palette from last render


--* Updated : 16/12/2004 ver 0.6
--*     - added: search color swatch by name and by color


--* Updated : 17/12/2004 ver 1.0 jumped to version 1 to be on the same leve with ACP
--*     - added: integration with ACP


--*
--*
--*  Plans for the future:                                                     *
--*  - this is going to be part of ACP. still need to work on the connection
--*    betweeen the two.
--*  - add a pantone color palette.
--*                                                                            *
--******************************************************************************
--* Isntallation:                                                              *
--* --------------                                                             *
--*                                                                            *
--*   after you run the script you will find it in the customize user interface*
--*   under "Os Tools" category.                                               *
--*                                                                            *
--******************************************************************************


macroScript colorPalette
	category:"Os Tools"
	tooltip:"Color Palette"
	buttontext:"Palette"
(
	global ro_ACPPalette
	try (destroyDialog ro_ACPPalette) catch()


	rollout ro_ACPPalette "Palette 1.1"
	(

	-- Global Variable Declerations
	--------------------------------------------------------------------------------
		global ACPPaletteSession


	-- Struct Declerations
	--------------------------------------------------------------------------------
		include "ACP\\Structs\\rect_struct.ms"
		include "ACP\\Structs\\color_struct.ms"
		include "ACP\\Structs\\palette_struct.ms"



	-- Local Rollout Declerations
	--------------------------------------------------------------------------------
		local ro_Tooltip

		rollout ro_Tooltip ""
		(
			label lblTooltip "" align:#left pos:[4,2]
			groupbox gbBorder "" pos:[0,-6] width:(ro_Tooltip.width) height:(ro_Tooltip.height+6)
		)




	-- Local Functions Declerations
	--------------------------------------------------------------------------------
		local createArrow



	-- Local Variable Declerations
	--------------------------------------------------------------------------------
		local ACPPalette = s_palette width:30 height:21 swatchHeight:11
		local paletteMapRect
		local allowEditing = false
		local rolloutHeight
		local findMenuCollapsed = true
		local findResults = #()
		local findNextIndex = 0
		local lastFindType, lastFindValue
		local mousePos = [-1, -1]
		local autoScrollClickPos
		local lastIndex = -1
		local tooltipVisible = false
		local inDragMode = false

		local icon_directory = getDir #scripts + "\\ACP\\Icons\\"


	-- User Interface
	--------------------------------------------------------------------------------
		button bn_ShadeWin "" width:(ro_ACPPalette.width+4) height:7 pos:[-2,-1] images:#((bitmap (ro_ACPPalette.width+4) 7 color:gray),undefined,1,1,1,1,1) tooltip:"Click to shade window."

		bitmap bmPalette "" bitmap:(ACPPalette.generatePalette()) offset:[0,5] align:#left
		label lblPalette "" pos:(bmPalette.pos + [38,3]) width:147 height:(bmPalette.bitmap.height)
		groupbox gbPalette "" pos:[5,5] width:210 height:(bmPalette.bitmap.height + 20)
		button bnOptionsMenu "" width:14 height:14 pos:[197, bmPalette.pos.y + 2] images:#((createArrow 5 9 black white offset:[1,0]), (createArrow 5 9 black white offset:[1,0]), 1, 1, 1, 1, 1)
		progressBar pbScroll color:(gray * 0.9) orient:#vertical pos:[198, bnOptionsMenu.pos.y + 16] width:12 height:(bmPalette.bitmap.height - 35)
		spinner spScroll "" fieldWidth:0 range:[0,100,0] type:#integer align:#right pos:[198,(pbScroll.pos.y + (bmPalette.bitmap.height - 33))]
--		spinner spScroll "" fieldWidth:0 range:[0,100,0] type:#integer align:#right pos:[185,(bmPalette.pos.y + (bmPalette.bitmap.height / 2) - 10)]

		groupbox gbActions "" pos:[5,(gbPalette.pos.y + gbPalette.height - 6)] width:210 height:28
			button bnCenter "" pos:(gbActions.pos + [5,10]) tooltip:"Center selected." width:14 height:14 images:#(icon_directory + "icon_center.png", icon_directory + "icon_center.png", 2, 1, 1, 2, 2)
			button bnMoveColorUp "" pos:(gbActions.pos + [110,10]) width:14 height:14 images:#(icon_directory + "icon_moveup.png", icon_directory + "icon_moveup.png", 2, 1, 1, 2, 2) enabled:false tooltip:"Move color swatch up."
			button bnMoveColorDown "" pos:(gbActions.pos + [130,10]) width:14 height:14 images:#(icon_directory + "icon_movedown.png", icon_directory + "icon_movedown.png", 2, 1, 1, 2, 2) enabled:false tooltip:"Move color swatch down."
			button bnAddColor "" pos:(gbActions.pos + [150,10]) width:14 height:14 images:#(icon_directory + "icon_new.png", icon_directory + "icon_new.png", 2, 1, 1, 2, 2) enabled:false tooltip:"New color swatch."
			button bnCopyColor "" pos:(gbActions.pos + [170,10]) width:14 height:14 images:#(icon_directory + "icon_copy.png", icon_directory + "icon_new.png", 2, 1, 1, 2, 2) enabled:false tooltip:"Duplicate selected color swatch."
			button bnDelColor "-" pos:(gbActions.pos + [190,10]) width:14 height:14 images:#(icon_directory + "icon_trash.png", icon_directory + "icon_trash.png", 2, 1, 1, 2, 2) enabled:false tooltip:"Delete selected color swatch."

		groupbox gbSelColor"Selected Color: " pos:[5,(gbActions.pos.y + gbActions.height + 5)] width:210 height:75
			edittext edName "Name:" pos:(gbSelColor.pos + [5,15]) align:#left
			colorPicker cpSwatchColor "" value:100. color:black height:17 fieldWidth:195 offset:[-5,0]
	--		progressBar cpSwatchColor "" value:100. color:black height:17 --fieldWidth:195 offset:[-5,0]
			label lblIndex "" offset:[0,-3]


		groupbox grpbxFind "    Find Color Swatch: " pos:[5, (gbSelColor.pos.y + gbSelColor.height + 5)] width:210 height:16
			button bnFindCollapse "" width:13 height:13 pos:[10, grpbxFind.pos.y] images:#((createArrow 5 9 black white offset:[1,0]), (createArrow 5 9 black white offset:[1,0]), 1, 1, 1, 1, 1) tooltip:"Expand/Collapse find options."
			label lblFindByName "By Name:" pos:(grpbxFind.pos + [5,15]) visible:false
			edittext edFindByName "" pos:(grpbxFind.pos + [5,30]) fieldWidth:160 visible:false
			button bnFindByNameGo "Find" pos:(grpbxFind.pos + [175,30]) width:30 height:16 visible:false
			label lblFindByColor "By Color:" pos:(grpbxFind.pos + [5,55]) visible:false
			colorPicker cpFindByColorCol "" pos:(grpbxFind.pos + [5,70]) fieldWidth:50 height:16 visible:false

			spinner spFindByColorAccuracy "Likeness: " type:#integer range:[0,100,70] fieldWidth:30 pos:(grpbxFind.pos + [70,70])
			label lblFindByColorAccuracyPer "%" pos:(grpbxFind.pos + [160,70])
			button bnFindByColorGo "Find" pos:(grpbxFind.pos + [175,70]) width:30 height:16 visible:false


		timer tmToolTipShow active:false interval:500
		timer tmAutoScroll active:false interval:100




	-- Local RCMenu Declerations
	--------------------------------------------------------------------------------

		local rc_Palette


		fn createRCMenu =
		(
			RCMenu rc_Palette
			(
				-- Local Function Declerations
				---------------------------------
				local filterEditable
				local filterLastRender


				-- User Interface
				-----------------------
				menuItem miAllowEditing "Allow Palette Editing" checked:ro_ACPPalette.allowEditing
				seperator menuSep01
				menuItem miResetPalette "Restore Default"
				menuItem miReplacePalette "Load Palette..."
				menuItem miAppendPalette "Merge Palette..."
				menuItem miRenderPalette "Get Palette From Last Render" filter:filterLastRender
				menuItem miSavePalette "Save Palette..."
				seperator menuSep02 filter:filterEditable
				subMenu "Sort Palette" filter:filterEditable
				(
					menuItem miSortName "Sort By Name" filter:filterEditable
					menuItem miSortRGB "Sort By RGB" filter:filterEditable
					menuItem miSortHSV "Sort By HSV" filter:filterEditable
					menuItem miSortLum "Sort By Luminance" filter:filterEditable
				)
				seperator menuSep03 filter:filterEditable
				menuItem miNewColor "New Color Swatch" filter:filterEditable
				menuItem miDuplicateSelected "Duplicate Selected Color Swatch" filter:filterEditable
				menuItem miDeleteSelected "Delete Selected Color Swatch" filter:filterEditable
				seperator menuSep04
				menuItem miClearPalette "Clear Palette"



				-- Functions
				------------------------
				fn filterEditable = ro_ACPPalette.allowEditing
				fn filterLastRender = (getLastRenderedImage()) != undefined

				fn updateMenuItems =
				(
					miAllowEditing.checked = ro_ACPPalette.allowEditing

					miNewColor.enabled = ro_ACPPalette.allowEditing
					miDuplicateSelected.enabled = ro_ACPPalette.ACPPalette.selected != undefined and ro_ACPPalette.allowEditing
					miDeleteSelected.enabled = ro_ACPPalette.ACPPalette.selected != undefined and ro_ACPPalette.allowEditing
				)


				-- Event Handlers
				--------------------
				on miAllowEditing picked do (
					ro_ACPPalette.allowEditing = not ro_ACPPalette.allowEditing
					miAllowEditing.checked = ro_ACPPalette.allowEditing
					updateMenuItems()
					ro_ACPPalette.updateUI()
				)

				on miResetPalette picked do (
					if queryBox "Restore default palette?" then (
						local fname = getDir #scripts + "\\ACP\\Palettes\\Default.aco"
						if not doesFileExist fname then (messageBox "Could not find default palette."; return())
						ro_ACPPalette.ACPPalette.loadPalette fname showWarnings:false appendPalette:false
						ro_ACPPalette.ACPPalette.selected = undefined
						ro_ACPPalette.updateUI()
						ro_ACPPalette.scrollPalette ro_ACPPalette.ACPPalette.scroll
					)
				)

				on miAppendPalette picked do (
					local fname = getOpenFileName caption:"Select palette to append..." types:"Adobe Photoshop Color File (*.aco)|*.aco|All Files (*.*)|*.*"
					ro_ACPPalette.ACPPalette.loadPalette fname showWarnings:true appendPalette:true
					ro_ACPPalette.updateUI()
					ro_ACPPalette.scrollPalette ro_ACPPalette.ACPPalette.scroll
				)

				on miReplacePalette picked do (
					local fname = getOpenFileName caption:"Select palette to load..." types:"Adobe Photoshop Color File (*.aco)|*.aco|All Files (*.*)|*.*"
					ro_ACPPalette.ACPPalette.loadPalette fname showWarnings:true appendPalette:false
					ro_ACPPalette.updateUI()
					ro_ACPPalette.scrollPalette ro_ACPPalette.ACPPalette.scroll
				)

				on miRenderPalette picked do (
					if queryBox "Replace current palette with palette from last render?" then (

					ro_ACPPalette.ACPPalette.getBitmapPalette (getLastRenderedImage())
						ro_ACPPalette.updateUI()
						ro_ACPPalette.scrollPalette ro_ACPPalette.ACPPalette.scroll
					)
				)

				on miSavePalette picked do (
					local fname = getSaveFileName caption:"Save palette as..." types:"Adobe Photoshop Color File (*.aco)|*.aco|All Files (*.*)|*.*"
					if fname != undefined then
						ro_ACPPalette.ACPPalette.savePalette fname
				)

				on miClearPalette picked do (
					if queryBox "Delete all color swatches in the palette?" then (

						ro_ACPPalette.ACPPalette.reset()
						ro_ACPPalette.ACPPalette.selected = undefined
						ro_ACPPalette.allowEditing = true
						ro_ACPPalette.updateUI()
						ro_ACPPalette.scrollPalette ro_ACPPalette.ACPPalette.scroll
					)
				)

				-- Sorts
				-------------------
				on miSortName picked do (
					ro_ACPPalette.ACPPalette.selected = undefined
					ro_ACPPalette.ACPPalette.sortByName()
					ro_ACPPalette.updateUI()
					ro_ACPPalette.scrollPalette ro_ACPPalette.ACPPalette.scroll
				)

				on miSortRGB picked do (
					ro_ACPPalette.ACPPalette.selected = undefined
					ro_ACPPalette.ACPPalette.sortByRGB()
					ro_ACPPalette.updateUI()
					ro_ACPPalette.scrollPalette ro_ACPPalette.ACPPalette.scroll
				)

				on miSortHSV picked do (
					ro_ACPPalette.ACPPalette.selected = undefined
					ro_ACPPalette.ACPPalette.sortByHSV()
					ro_ACPPalette.updateUI()
					ro_ACPPalette.scrollPalette ro_ACPPalette.ACPPalette.scroll
				)

				on miSortLum picked do (
					ro_ACPPalette.ACPPalette.selected = undefined
					ro_ACPPalette.ACPPalette.sortByLuminance()
					ro_ACPPalette.updateUI()
					ro_ACPPalette.scrollPalette ro_ACPPalette.ACPPalette.scroll
				)

				-- Color Actions
				-------------------
				on miNewColor picked do (
					ro_ACPPalette.bnAddColor.pressed()
				)

				on miDuplicateSelected picked do (
					ro_ACPPalette.bnCopyColor.pressed()
				)

				on miDeleteSelected picked do (
					ro_ACPPalette.bnDelColor.pressed()
				)

				on rc_Palette open do (
					updateMenuItems()
				)
			) -- end of rc_Palette RCMenu
		)





	-- Functions
	--------------------------------------------------------------------------------
		fn createTooltip pos text =
		(
			try(destroyDialog ro_Tooltip)catch()
			local size = (getTextExtent text) + [10,5]
			createDialog ro_Tooltip width:size.x height:size.y pos:(pos + [20,35]) bgColor:(color 255 255 180) style:#()--#style_border)
			ro_Tooltip.lblTooltip.text = text
		)


		fn createArrow width height color bgColor direction:#right offset:[0,0] =
		(
			fn round n = floor (n + 0.5)

			local bm = bitmap (width + offset.x) (height + offset.y) color:bgColor
			local arr = #()
			for y = 0 to (height - 1) do (
				case direction of (

					#right: (
						local yy = if y <= height/2. then ((y + 1.) / height) else ((height - y) as float / height)
						local w = round (width * yy * 2)
						arr = for x = 1 to width collect if (x <= w) then color else bgColor
					)
					#left: (
						local yy = if y <= height/2. then ((y + 1.) / height) else ((height - y) as float / height)
						local w = round (width * yy * 2)
						arr = for x = width to 1 by -1 collect if (x <= w) then color else bgColor
					)
					#down: (
						local yy = y as float / height / 2.
						local w = round (width * yy)
						arr = for x = 1 to width collect if (x > w) and (x <= (width - w)) then color else bgColor
					)
					#up: (
						local yy = (y + 1.) / height / 2.
						local w = round (width * (0.5 - yy))
						arr = for x = 1 to width collect if (x > w) and (x <= (width - w)) then color else bgColor
					)
				)
				setPixels bm [offset.x,y + offset.y] arr
			)
			bm
		)



		fn updateSwatchesNameList =
		(
			local txt = stringStream ""
			for i = 0 to (ACPPalette.height - 1) do (
				ii = ACPPalette.scroll + i
				sw = if (ii <= ACPPalette.swatches.count) and (ii > 0) then ACPPalette.swatches[ii] else s_swatch()
				format "%" sw.name to:txt
				if i < (ACPPalette.height - 1) then
					format "\n" to:txt
			)
			lblPalette.text = txt as string
		)
		
		
		fn scrollPalette scroll center:false =
		(
			ACPPalette.scroll = scroll
			if center then (
				ACPPalette.centerSelected()
				scroll = ACPPalette.scroll
			) else
				ACPPalette.generatePalette()
			ACPPalette.markSelected()
			bmPalette.bitmap = ACPPalette.bm
			pbScroll.value = scroll / (spScroll.range[2] as float) * 100.

			updateSwatchesNameList()
			spScroll.value = scroll
		)


		fn colorTooltip =
		(
			local index = ACPPalette.getSwatchIndex (mousePos - bmPalette.pos)
			if not ACPPalette.isValidIndex index then
			(
				try(destroyDialog ro_Tooltip)catch()
				tooltipVisible = false
				tmToolTipShow.active = false
				return()
			)
			local col = ACPPalette.getSwatchColor index
			if col == undefined then col = balck
			col = #(col.r as integer, col.g as integer, col.b as integer)
			local str = ACPPalette.getSwatchName index + " (" + col[1] as string + "," + col[2] as string + "," + col[3] as string + ")"
			createTooltip (mousePos + (GetDialogPos ro_ACPPalette)) str
			tooltipVisible = true
			tmToolTipShow.active = false
		)



		-- Integration with ACP
		fn updateColorPicker =
		(
			if ACPPalette.selected == undefined then return()
			try (
				ACPClipBoard = ACPPalette.getSwatchColor ACPPalette.selected
				ro_ACPPhotoshop.updateFromPalette()
			)catch()
		)

		fn updateFromColorPicker =
		(
			if ACPPalette.selected != undefined and allowEditing then (
				ACPPalette.setSwatchColor ACPPalette.selected ACPClipBoard
				scrollPalette ACPPalette.scroll
				local col = ACPPalette.getSwatchColor ACPPalette.selected
				if col != undefined then (
					cpSwatchColor.color = col
					col = #((col.r as integer), (col.g as integer), (col.b as integer))
					lblIndex.text = ("Index: " + ACPPalette.selected as string + "   (" + col[1] as string + "," + col[2] as string + "," + col[3] as string + ")")
				)
			)
		)



		-- Update UI Elements
		fn updateUI =
		(
			spScroll.range = [1,(ACPPalette.swatches.count - ACPPalette.height + 1),spScroll.value]

			grpbxFind.height = if findMenuCollapsed then 16 else 95
			local findCollapseImg = if findMenuCollapsed then (createArrow 5 9 black white offset:[1,0] direction:#right) else (createArrow 9 5 black white offset:[0,0] direction:#down)
			bnFindCollapse.images = #(findCollapseImg , findCollapseImg , 1, 1, 1, 1, 1)
			lblFindByName.visible = not findMenuCollapsed
			edFindByName.visible = not findMenuCollapsed
			bnFindByNameGo.visible = not findMenuCollapsed
			lblFindByColor.visible = not findMenuCollapsed
			cpFindByColorCol.visible = not findMenuCollapsed
			spFindByColorAccuracy.visible = not findMenuCollapsed
			lblFindByColorAccuracyPer.visible = not findMenuCollapsed
			bnFindByColorGo.visible = not findMenuCollapsed

			bnCenter.enabled = ACPPalette.selected != undefined
			bnMoveColorUp.enabled = ACPPalette.selected != undefined and allowEditing
			bnMoveColorDown.enabled = ACPPalette.selected != undefined and allowEditing
			bnAddColor.enabled = allowEditing
			bnCopyColor.enabled = ACPPalette.selected != undefined and allowEditing
			bnDelColor.enabled = ACPPalette.selected != undefined and allowEditing
			cpSwatchColor.enabled = ACPPalette.selected != undefined and allowEditing
			edName.enabled = ACPPalette.selected != undefined and allowEditing

			if ACPPalette.selected == undefined then (
				cpSwatchColor.color = black
				lblIndex.text = ""
				edName.text = ""
			) else (
				local col = ACPPalette.getSwatchColor ACPPalette.selected
				if col == undefined then col = black
				cpSwatchColor.color = col
				col = #((col.r as integer), (col.g as integer), (col.b as integer))
				lblIndex.text = ("Index: " + ACPPalette.selected as string + "   (" + col[1] as string + "," + col[2] as string + "," + col[3] as string + ")")
				local txt = ACPPalette.getSwatchName ACPPalette.selected
				edName.text = if txt != undefeind then txt else ""
			)
		)


		-- Initialize
		fn init =
		(
			createRCMenu()
			paletteMapRect = s_rect (bmPalette.pos) ((bmPalette.pos+[bmPalette.bitmap.width,bmPalette.bitmap.height])-[1,1])
			ACPPalette.swatchHeight = (getTextExtent "X").y - ACPPalette.spacing
			if ACPPaletteSession != undefined then
				ACPPalette = ACPPaletteSession
			else
				ACPPalette.loadPalette (getDir #scripts + "\\ACP\\Palettes\\Default.aco") showWarnings:true
			updateUI()
			bmPalette.bitmap = ACPPalette.generatePalette()
			updateSwatchesNameList()
			try(ro_ACPPhotoshop.psPalette.state = true)catch()
			freeSceneBitmaps()
			gc light:true
		)


		-- Clean up the mess when closing...
		fn done =
		(
			ACPPaletteSession = ACPPalette
			try(destroyDialog ro_Tooltip)catch()
			try(ro_ACPPhotoshop.psPalette.state = false)catch()
			freeSceneBitmaps()
			ro_ACPPalette = undefined
			gc light:true
		)

		
		fn openDialog =
		(
			createDialog ro_ACPPalette width:220 height:430
		)


	-- Event Handlers
	--------------------------------------------------------------------------------
		on bn_ShadeWin pressed do (
			if ro_ACPPalette.height >= 8 then (
				rolloutHeight = ro_ACPPalette.height
				ro_ACPPalette.height = 7
			) else
				ro_ACPPalette.height = rolloutHeight
		)

		on bnCenter pressed do (
			scrollPalette 1 center:true
		)


		on bnMoveColorUp pressed do (
			if ACPPalette.selected != undefined and ACPPalette.selected > 1 then (
				local i = ACPPalette.selected
				ACPPalette.moveSwatch i (i - 1)
				ACPPalette.selected = i - 1
				updateUI()
				scrollPalette 1 center:true
			)
		)

		on bnMoveColorDown pressed do (
			if ACPPalette.selected != undefined and ACPPalette.selected < ACPPalette.swatches.count then (
				local i = ACPPalette.selected
				ACPPalette.moveSwatch i (i + 1)
				ACPPalette.selected = i + 1
				updateUI()
				scrollPalette 1 center:true
			)
		)


		on bnAddColor pressed do (
			local col = if isKindOf ACPClipBoard color then ACPClipBoard else black
			ACPPalette.addSwatch "New Swatch" col
			ACPPalette.selected = ACPPalette.swatches.count
			updateUI()
			scrollPalette 1 center:true
		)

		on bnCopyColor pressed do (
			local col = ACPPalette.getSwatchColor ACPPalette.selected
			local name = ACPPalette.getSwatchName ACPPalette.selected
			ACPPalette.addSwatch (name + " Copy") col
			ACPPalette.selected = ACPPalette.swatches.count
			updateUI()
			scrollPalette 1 center:true
		)

		on bnDelColor pressed do (
			if ACPPalette.selected != undefined then (
				if queryBox ("Delete color swatch \"" + (ACPPalette.getSwatchName ACPPalette.selected) + "\"?") then (
					ACPPalette.deleteSwatch ACPPalette.selected
					ACPPalette.selected = undefined
					updateUI()
					scrollPalette (amin ACPPalette.scroll spScroll.range[2])
				)
			)
		)

		on edName entered text do (
			ACPPalette.setSwatchName ACPPalette.selected text
		)


		on cpSwatchColor changed value do (
			ACPPalette.setSwatchColor ACPPalette.selected value
			scrollPalette ACPPalette.scroll
		)


		-- Scroll handlers
		on spScroll changed val do (
			scrollPalette val
		)
		on spScroll buttonup do (gc light:true)

		on pbScroll clicked val do (
			scrollPalette (amax (val / 100. * spScroll.range[2]) 1)
			gc light:true
		)


		-- find menu handlers
		on bnFindCollapse pressed do (
			if findMenuCollapsed then (
				ro_ACPPalette.height += 77
			) else (
				ro_ACPPalette.height -= 77
			)
			findMenuCollapsed = not findMenuCollapsed
			updateUI()
		)

		on edFindByName entered txt do (
			findNextIndex = 0
		)

		on bnFindByNameGo pressed do (
			if edFindByName.text != "" then (
				if lastFindType != #name or lastFindValue != edFindByName.text then (
					lastFindType = #name
					lastFindValue = edFindByName.text
					findNextIndex = 0
				)
				findResults = ACPPalette.searchByName edFindByName.text
				if findResults.count == 0 then (
					messageBox "No swatches with matching names were found." title:"Palette - Find By Name" beep:false
				) else (
					if findNextIndex < findResults.count then (
						findNextIndex += 1
						ACPPalette.selected = findResults[findNextIndex].index
						updateUI()
						scrollPalette 1 center:true
						updateColorPicker()
					) else (
						if queryBox "Reached the end of the search results.\nDo you wish to go back to the first result?" then (
							findNextIndex = 1
							ACPPalette.selected = findResults[findNextIndex].index
							updateUI()
							scrollPalette 1 center:true
							updateColorPicker()
						)
					)
				)
			)
		)


		on cpFindByColorCol changed value do (
			findNextIndex = 0
		)


		on spFindByColorAccuracy changed value do (
			findNextIndex = 0
		)


		on bnFindByColorGo pressed do (
			if lastFindType != #color or lastFindValue != cpFindByColorCol.color then (
				lastFindType = #color
				lastFindValue = cpFindByColorCol.color
				findNextIndex = 0
			)
			findResults = ACPPalette.searchByColor cpFindByColorCol.color likeness:(spFindByColorAccuracy.value / 100.)
			if findResults.count == 0 then (
				messageBox "No swatches with matching colors were found." title:"Palette - Find By Color" beep:false
			) else (

				if findNextIndex < findResults.count then (
					findNextIndex += 1
					ACPPalette.selected = findResults[findNextIndex].index
					updateUI()
					scrollPalette 1 center:true
					updateColorPicker()
				) else (
					if queryBox "Reached the end of the search results.\nDo you wish to go back to the first result?" then (
						findNextIndex = 1
						ACPPalette.selected = findResults[findNextIndex].index
						updateUI()
						scrollPalette 1 center:true
						updateColorPicker()
					)
				)
			)
		)

		-- timers
		on tmToolTipShow tick do (
			colorTooltip()
		)

		on tmAutoScroll tick do (
			local scroll = ACPPalette.scroll
			if mousePos.y < (autoScrollClickPos.y - 10) then (
				scroll += ((abs(mousePos.y - autoScrollClickPos.y) - 10) / 20) as integer
				if scroll > (ACPPalette.swatches.count - ACPPalette.height + 1) then
					scroll = (ACPPalette.swatches.count - ACPPalette.height + 1)
			) else if mousePos.y > (autoScrollClickPos.y + 10) then (
				scroll -= ((abs(mousePos.y - autoScrollClickPos.y) - 10) / 20) as integer
				if scroll < 1 then
					scroll = 1
			)
			scrollPalette scroll
		)


		-- mouse handlers

		on ro_ACPPalette mousemove pos do (
			local index = (ACPPalette.getSwatchIndex (mousePos - bmPalette.pos))
			-- hide tooltip
			if (mousePos != pos and lastIndex != index) or not (paletteMapRect.isInside pos) then (
				if ro_Tooltip.open then (
					destroyDialog ro_Tooltip
					setFocus ro_ACPPalette.bmPalette
				)
				tmToolTipShow.active = false
				tooltipVisible = false
			)
			-- start the tooltip timer
			if not tooltipVisible and (paletteMapRect.isInside pos) then (
				tmToolTipShow.active = true
			)

			-- store the current index and mouse pos
			lastIndex = index
			mousePos = pos

		)

		on ro_ACPPalette mbuttondown pos do (
			if ro_Tooltip.open then
				destroyDialog ro_Tooltip
			if (paletteMapRect.isInside pos) then (
				inDragMode = true
				autoScrollClickPos = pos
				tmAutoScroll.active = true
			)
		)

		on ro_ACPPalette mbuttonup pos do (
			inDragMode = false
			tmAutoScroll.active = false
		)

		on ro_ACPPalette lbuttondown pos do (
			if ro_Tooltip.open then
				destroyDialog ro_Tooltip
			if (paletteMapRect.isInside pos) then (
				local i = (ACPPalette.getSwatchIndex (pos - bmPalette.pos))
				if ACPPalette.isValidIndex i then (
					ACPPalette.select i
					updateColorPicker()
				) else
					ACPPalette.deselect
				bmPalette.bitmap = ACPPalette.bm
				updateUI()
				gc light:true
			)
		)


		on ro_ACPPalette rbuttondown pos do (
			if ro_Tooltip.open then
				destroyDialog ro_Tooltip
			inDragMode = false
			tmAutoScroll.active = false
			popUpMenu rc_Palette pos:(pos - [2,2]) rollout:ro_ACPPalette align:#align_topleft
		)


		on bnOptionsMenu pressed do (
			popUpMenu rc_Palette pos:(bnOptionsMenu.pos + [15,0]) rollout:ro_ACPPalette align:#align_topleft
		)


		on ro_ACPPalette open do init()
		on ro_ACPPalette close do done()
	)


	ro_ACPPalette.openDialog()
) -- end of macroScript


